/* ***********************************************************
 * This file was automatically generated on 2012-07-13.      *
 *                                                           *
 * If you have a bugfix for this file and want to commit it, *
 * please fix the bug in the generator. You can find a link  *
 * to the generator git on tinkerforge.com                   *
 *************************************************************/

package com.tinkerforge;

import java.nio.ByteBuffer;
import java.nio.ByteOrder;
import java.util.concurrent.TimeUnit;


/**
 * Device for controlling Stacks and four Bricklets
 */
public class BrickMaster extends Device {
	private final static byte FUNCTION_GET_STACK_VOLTAGE = (byte)1;
	private final static byte FUNCTION_GET_STACK_CURRENT = (byte)2;
	private final static byte FUNCTION_SET_EXTENSION_TYPE = (byte)3;
	private final static byte FUNCTION_GET_EXTENSION_TYPE = (byte)4;
	private final static byte FUNCTION_IS_CHIBI_PRESENT = (byte)5;
	private final static byte FUNCTION_SET_CHIBI_ADDRESS = (byte)6;
	private final static byte FUNCTION_GET_CHIBI_ADDRESS = (byte)7;
	private final static byte FUNCTION_SET_CHIBI_MASTER_ADDRESS = (byte)8;
	private final static byte FUNCTION_GET_CHIBI_MASTER_ADDRESS = (byte)9;
	private final static byte FUNCTION_SET_CHIBI_SLAVE_ADDRESS = (byte)10;
	private final static byte FUNCTION_GET_CHIBI_SLAVE_ADDRESS = (byte)11;
	private final static byte FUNCTION_GET_CHIBI_SIGNAL_STRENGTH = (byte)12;
	private final static byte FUNCTION_GET_CHIBI_ERROR_LOG = (byte)13;
	private final static byte FUNCTION_SET_CHIBI_FREQUENCY = (byte)14;
	private final static byte FUNCTION_GET_CHIBI_FREQUENCY = (byte)15;
	private final static byte FUNCTION_SET_CHIBI_CHANNEL = (byte)16;
	private final static byte FUNCTION_GET_CHIBI_CHANNEL = (byte)17;
	private final static byte FUNCTION_IS_RS485_PRESENT = (byte)18;
	private final static byte FUNCTION_SET_RS485_ADDRESS = (byte)19;
	private final static byte FUNCTION_GET_RS485_ADDRESS = (byte)20;
	private final static byte FUNCTION_SET_RS485_SLAVE_ADDRESS = (byte)21;
	private final static byte FUNCTION_GET_RS485_SLAVE_ADDRESS = (byte)22;
	private final static byte FUNCTION_GET_RS485_ERROR_LOG = (byte)23;
	private final static byte FUNCTION_SET_RS485_CONFIGURATION = (byte)24;
	private final static byte FUNCTION_GET_RS485_CONFIGURATION = (byte)25;
	private final static byte FUNCTION_RESET = (byte)243;
	private final static byte FUNCTION_GET_CHIP_TEMPERATURE = (byte)242;

	public class ChibiErrorLog {
		public int underrun;
		public int crcError;
		public int noAck;
		public int overflow;

		public String toString() {
			return "[" + "underrun = " + underrun + ", " + "crcError = " + crcError + ", " + "noAck = " + noAck + ", " + "overflow = " + overflow + "]";
		}
	}

	public class RS485Configuration {
		public long speed;
		public char parity;
		public short stopbits;

		public String toString() {
			return "[" + "speed = " + speed + ", " + "parity = " + parity + ", " + "stopbits = " + stopbits + "]";
		}
	}

	/**
	 * Creates an object with the unique device ID \c uid. This object can
	 * then be added to the IP connection.
	 */
	public BrickMaster(String uid) {
		super(uid);

		expectedName = "Master Brick";

		bindingVersion[0] = 1;
		bindingVersion[1] = 2;
		bindingVersion[2] = 1;
	}

	/**
	 * Returns the stack voltage in mV. The stack voltage is the
	 * voltage that is supplied via the stack, i.e. it is given by a 
	 * Step-Down or Step-Up Power Supply.
	 */
	public int getStackVoltage() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_STACK_VOLTAGE, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_STACK_VOLTAGE, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getStackVoltage in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int voltage = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return voltage;
	}

	/**
	 * Returns the stack current in mA. The stack current is the
	 * current that is drawn via the stack, i.e. it is given by a
	 * Step-Down or Step-Up Power Supply.
	 */
	public int getStackCurrent() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_STACK_CURRENT, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_STACK_CURRENT, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getStackCurrent in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int current = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return current;
	}

	/**
	 * Writes the extension type to the EEPROM of a specified extension. 
	 * The extension is either 0 or 1 (0 is the on the bottom, 1 is the on on top, 
	 * if only one extension is present use 0).
	 * 
	 * Possible extension types:
	 * 
	 * \verbatim
	 *  "Type", "Description"
	 * 
	 *  "1",    "Chibi"
	 *  "2",    "RS485"
	 * \endverbatim
	 * 
	 * The extension type is already set when bought and it can be set with the 
	 * Brick Viewer, it is unlikely that you need this function.
	 * 
	 * The value will be saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 */
	public void setExtensionType(short extension, long exttype)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_EXTENSION_TYPE, (short)9);
		bb.put((byte)extension);
		bb.putInt((int)exttype);

		ipcon.write(this, bb, FUNCTION_SET_EXTENSION_TYPE, false);
	}

	/**
	 * Returns the extension type for a given extension as set by 
	 * {@link com.tinkerforge.BrickMaster.setExtensionType}.
	 */
	public long getExtensionType(short extension) throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_EXTENSION_TYPE, (short)5);
		bb.put((byte)extension);

		ipcon.write(this, bb, FUNCTION_GET_EXTENSION_TYPE, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getExtensionType in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		long exttype = IPConnection.unsignedInt(bb.getInt());

		semaphoreWrite.release();
		return exttype;
	}

	/**
	 * Returns true if a Chibi Extension is available to be used by the Master.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public boolean isChibiPresent() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_IS_CHIBI_PRESENT, (short)4);

		ipcon.write(this, bb, FUNCTION_IS_CHIBI_PRESENT, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for isChibiPresent in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		boolean present = (bb.get()) != 0;

		semaphoreWrite.release();
		return present;
	}

	/**
	 * Sets the address (1-255) belonging to the Chibi Extension.
	 * 
	 * It is possible to set the address with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public void setChibiAddress(short address)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CHIBI_ADDRESS, (short)5);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_ADDRESS, false);
	}

	/**
	 * Returns the address as set by {@link com.tinkerforge.BrickMaster.setChibiAddress}.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiAddress() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_ADDRESS, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_ADDRESS, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Sets the address (1-255) of the Chibi Master. This address is used if the
	 * Chibi Extension is used as slave (i.e. it does not have a USB connection).
	 * 
	 * It is possible to set the address with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public void setChibiMasterAddress(short address)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CHIBI_MASTER_ADDRESS, (short)5);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_MASTER_ADDRESS, false);
	}

	/**
	 * Returns the address as set by {@link com.tinkerforge.BrickMaster.setChibiMasterAddress}.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiMasterAddress() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_MASTER_ADDRESS, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_MASTER_ADDRESS, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiMasterAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Sets up to 255 slave addresses. The address numeration has to be used
	 * ascending from 0. For example: If you use the Chibi Extension in Master mode
	 * (i.e. the stack has an USB connection) and you want to talk to three other
	 * Chibi stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
	 * (1, 23) and (2, 42)".
	 * 
	 * It is possible to set the addresses with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, they don't
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public void setChibiSlaveAddress(short num, short address)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CHIBI_SLAVE_ADDRESS, (short)6);
		bb.put((byte)num);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_SLAVE_ADDRESS, false);
	}

	/**
	 * Returns the slave address for a given num as set by 
	 * {@link com.tinkerforge.BrickMaster.setChibiSlaveAddress}.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiSlaveAddress(short num) throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_SLAVE_ADDRESS, (short)5);
		bb.put((byte)num);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_SLAVE_ADDRESS, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiSlaveAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Returns the signal strength in dBm. The signal strength updates every time a
	 * packet is received.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiSignalStrength() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_SIGNAL_STRENGTH, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiSignalStrength in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short signalStrength = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return signalStrength;
	}

	/**
	 * Returns underrun, CRC error, no ACK and overflow error counts of the Chibi
	 * communication. If these errors start rising, it is likely that either the
	 * distance between two Chibi stacks is becoming too big or there are
	 * interferences.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public ChibiErrorLog getChibiErrorLog() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_ERROR_LOG, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_ERROR_LOG, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiErrorLog in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		ChibiErrorLog obj = new ChibiErrorLog();
		obj.underrun = IPConnection.unsignedShort(bb.getShort());
		obj.crcError = IPConnection.unsignedShort(bb.getShort());
		obj.noAck = IPConnection.unsignedShort(bb.getShort());
		obj.overflow = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return obj;
	}

	/**
	 * Sets the Chibi frequency range for the Chibi Extension. Possible values are:
	 * 
	 * \verbatim
	 *  "Type", "Description"
	 * 
	 *  "0",    "OQPSK 868Mhz (Europe)"
	 *  "1",    "OQPSK 915Mhz (US)"
	 *  "2",    "OQPSK 780Mhz (China)"
	 *  "3",    "BPSK40 915Mhz"
	 * \endverbatim
	 * 
	 * It is possible to set the frequency with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public void setChibiFrequency(short frequency)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CHIBI_FREQUENCY, (short)5);
		bb.put((byte)frequency);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_FREQUENCY, false);
	}

	/**
	 * Returns the frequency value as set by {@link com.tinkerforge.BrickMaster.setChibiFrequency}.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiFrequency() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_FREQUENCY, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_FREQUENCY, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiFrequency in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short frequency = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return frequency;
	}

	/**
	 * Sets the channel used by the Chibi Extension. Possible channels are
	 * different for different frequencies:
	 * 
	 * \verbatim
	 *  "Frequency",             "Possible Channels"
	 * 
	 *  "OQPSK 868Mhz (Europe)", "0"
	 *  "OQPSK 915Mhz (US)",     "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
	 *  "OQPSK 780Mhz (China)",  "0, 1, 2, 3"
	 *  "BPSK40 915Mhz",         "1, 2, 3, 4, 5, 6, 7, 8, 9, 10"
	 * \endverbatim
	 * 
	 * It is possible to set the frequency with the Brick Viewer and it will be 
	 * saved in the EEPROM of the Chibi Extension, it does not
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public void setChibiChannel(short channel)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_CHIBI_CHANNEL, (short)5);
		bb.put((byte)channel);

		ipcon.write(this, bb, FUNCTION_SET_CHIBI_CHANNEL, false);
	}

	/**
	 * Returns the channel as set by {@link com.tinkerforge.BrickMaster.setChibiChannel}.
	 * 
	 * .. versionadded:: 1.1.0
	 */
	public short getChibiChannel() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIBI_CHANNEL, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIBI_CHANNEL, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChibiChannel in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short channel = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return channel;
	}

	/**
	 * Returns true if a RS485 Extension is available to be used by the Master.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public boolean isRS485Present() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_IS_RS485_PRESENT, (short)4);

		ipcon.write(this, bb, FUNCTION_IS_RS485_PRESENT, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for isRS485Present in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		boolean present = (bb.get()) != 0;

		semaphoreWrite.release();
		return present;
	}

	/**
	 * Sets the address (1-255) belonging to the RS485 Extension.
	 * 
	 * Set to 0 if the RS485 Extension should be the RS485 Master (i.e.
	 * connected to a PC via USB).
	 * 
	 * It is possible to set the address with the Brick Viewer and it will be 
	 * saved in the EEPROM of the RS485 Extension, it does not
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public void setRS485Address(short address)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_RS485_ADDRESS, (short)5);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_RS485_ADDRESS, false);
	}

	/**
	 * Returns the address as set by {@link com.tinkerforge.BrickMaster.setRS485Address}.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public short getRS485Address() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_RS485_ADDRESS, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_RS485_ADDRESS, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getRS485Address in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Sets up to 255 slave addresses. The address numeration has to be used
	 * ascending from 0. For example: If you use the RS485 Extension in Master mode
	 * (i.e. the stack has an USB connection) and you want to talk to three other
	 * RS485 stacks with the IDs 17, 23, and 42, you should call with "(0, 17),
	 * (1, 23) and (2, 42)".
	 * 
	 * It is possible to set the addresses with the Brick Viewer and it will be 
	 * saved in the EEPROM of the RS485 Extension, they don't
	 * have to be set on every startup.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public void setRS485SlaveAddress(short num, short address)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_RS485_SLAVE_ADDRESS, (short)6);
		bb.put((byte)num);
		bb.put((byte)address);

		ipcon.write(this, bb, FUNCTION_SET_RS485_SLAVE_ADDRESS, false);
	}

	/**
	 * Returns the slave address for a given num as set by 
	 * {@link com.tinkerforge.BrickMaster.setRS485SlaveAddress}.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public short getRS485SlaveAddress(short num) throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_RS485_SLAVE_ADDRESS, (short)5);
		bb.put((byte)num);

		ipcon.write(this, bb, FUNCTION_GET_RS485_SLAVE_ADDRESS, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getRS485SlaveAddress in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short address = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return address;
	}

	/**
	 * Returns CRC error counts of the RS485 communication.
	 * If this counter starts rising, it is likely that the distance
	 * between the RS485 nodes is too big or there is some kind of
	 * interference.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public int getRS485ErrorLog() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_RS485_ERROR_LOG, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_RS485_ERROR_LOG, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getRS485ErrorLog in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		int crcError = IPConnection.unsignedShort(bb.getShort());

		semaphoreWrite.release();
		return crcError;
	}

	/**
	 * Sets the configuration of the RS485 extension. Speed is given in baud. The
	 * Master Brick will try to match the given baud rate as exactly as possible.
	 * The maximum recommended baud rate is 2000000 (2Mbit).
	 * Possible values for parity are 'n' (none), 'e' (even) and 'o' (odd).
	 * Possible values for stopbits are 1 and 2.
	 * 
	 * If your RS485 is unstable (lost messages etc), the first thing you should
	 * try is to decrease the speed. On very large bus (e.g. 1km), you probably
	 * should use a value in the range of 100khz.
	 * 
	 * The values are stored in the EEPROM and only applied on startup. That means
	 * you have to restart the Master Brick after configuration.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public void setRS485Configuration(long speed, char parity, short stopbits)  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_SET_RS485_CONFIGURATION, (short)10);
		bb.putInt((int)speed);
		bb.put((byte)parity);
		bb.put((byte)stopbits);

		ipcon.write(this, bb, FUNCTION_SET_RS485_CONFIGURATION, false);
	}

	/**
	 * Returns the configuration as set by {@link com.tinkerforge.BrickMaster.setRS485Configuration}.
	 * 
	 * .. versionadded:: 1.2.0
	 */
	public RS485Configuration getRS485Configuration() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_RS485_CONFIGURATION, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_RS485_CONFIGURATION, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getRS485Configuration in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		RS485Configuration obj = new RS485Configuration();
		obj.speed = IPConnection.unsignedInt(bb.getInt());
		obj.parity = (char)(bb.get());
		obj.stopbits = IPConnection.unsignedByte(bb.get());

		semaphoreWrite.release();
		return obj;
	}

	/**
	 * Calling this function will reset the Brick. Calling this function 
	 * on a Brick inside of a stack will reset the whole stack.
	 * 
	 * After a reset you have to create new device objects, 
	 * calling functions on the existing ones will result in 
	 * undefined behavior!
	 */
	public void reset()  {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_RESET, (short)4);

		ipcon.write(this, bb, FUNCTION_RESET, false);
	}

	/**
	 * Returns the temperature in °C/10 as measured inside the microcontroller. The
	 * value returned is not the ambient temperature! 
	 * 
	 * The temperature has an accuracy of +-15%. Practically it is only usefull as
	 * an indicator for temperature changes.
	 */
	public short getChipTemperature() throws IPConnection.TimeoutException {
		ByteBuffer bb = IPConnection.createRequestBuffer((byte)stackID, FUNCTION_GET_CHIP_TEMPERATURE, (short)4);

		ipcon.write(this, bb, FUNCTION_GET_CHIP_TEMPERATURE, true);

		byte[] response = null;
		try {
			response = responseQueue.poll(IPConnection.RESPONSE_TIMEOUT, TimeUnit.MILLISECONDS);
			if(response == null) {
				throw new IPConnection.TimeoutException("Did not receive response for getChipTemperature in time");
			}
		} catch (InterruptedException e) {
			e.printStackTrace();
		}

		bb = ByteBuffer.wrap(response, 4, response.length - 4);
		bb.order(ByteOrder.LITTLE_ENDIAN);

		short temperature = (bb.getShort());

		semaphoreWrite.release();
		return temperature;
	}
}